// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "AssetLoader.hpp"

using namespace Dawn;

AssetLoader::AssetLoader(std::string fileName) {
  this->fileName = fileName;
  this->handle = nullptr;
}

void AssetLoader::open() {
  std::string pathFull = DAWN_ASSET_BUILD_PREFIX + this->fileName;
  this->handle = fopen(pathFull.c_str(), "rb");
  if(this->handle == NULL || this->handle == nullptr) {
    throw "Failed to open file handle for " + this->fileName;
  }
}

int32_t AssetLoader::close() {
  int32_t ret = fclose(this->handle);
  this->handle = nullptr;
  return ret;
}

size_t AssetLoader::read(uint8_t *buffer, size_t size) {
  return fread(buffer, 1, size, this->handle);
}

int32_t AssetLoader::end() {
  return fseek(this->handle, 0, SEEK_END);
}

size_t AssetLoader::skip(size_t n) {
  return fseek(this->handle, n, SEEK_CUR);
}

int32_t AssetLoader::rewind() {
  return fseek(this->handle, 0, SEEK_SET);
}

size_t AssetLoader::getPosition() {
  return ftell(this->handle);
}

size_t AssetLoader::loadRaw(uint8_t **buffer) {
  size_t length, read;

  // Open a buffer.
  this->open();

  // Read the count of bytes in the file
  this->end();
  length = this->getPosition();

  // Are we only reading the size?
  if(buffer == nullptr) {
    this->close();
    return length;
  }

  // Reset to start
  this->rewind();

  // Read the string then close the file handle.
  *buffer = static_cast<uint8_t *>(malloc(sizeof(uint8_t) * length));
  read = this->read(*buffer, length);
  this->close();

  // Did we read successfully?
  if(read < length) {
    throw "Failed to read all bytes of " + this->fileName;
  }

  // Read successfully, return the read bytes.
  return read;
}

AssetLoader::~AssetLoader() {
  if(this->handle != nullptr) {
    this->close();
    this->handle = nullptr;
  }
}