// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "asset/Asset.hpp"

namespace Dawn {
  class AssetManager {
    private:
      /** List of pointers to assets, mapped by their asset key. */
      std::map<std::string, std::shared_ptr<Asset>> assets;
      std::map<std::string, std::shared_ptr<Asset>> assetsNotLoaded;

    public:
      void init();
      void update();

      /**
       * Creates and queue an asset to load.
       * 
       * @param name Name of the asset to load.
       * @return The asset element to be loaded.
       */
      template<class T>
      std::shared_ptr<T> load(std::string name) {
        auto existing = this->assets.find(name);
        if(existing != this->assets.end()) {
          return std::dynamic_pointer_cast<T>(existing->second);
        }
        auto asset = std::make_shared<T>(*this, name);
        this->assets[name] = asset;
        this->assetsNotLoaded[name] = asset;
        return asset;
      }
  };
}