// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TrueTypeAsset.hpp"

using namespace Dawn;

TrueTypeAsset::TrueTypeAsset(AssetManager &assMan, std::string name) :
  Asset(assMan, name),
  loader(name + ".truetype")
{
}

void TrueTypeAsset::updateSync() {
  if(this->state != 0x04) return;
  this->font.texture.setSize(this->width, this->height);
  this->font.texture.buffer(this->pixels);
  auto i = this->pixels;
  memoryCopy(
    this->characterData,
    this->font.characterData,
    sizeof(truetypechar_t) * TRUETYPE_NUM_CHARS
  );
  this->state = 0x05;
  this->loaded = true;
}

void TrueTypeAsset::updateAsync() {
  int32_t fontSize;
  size_t i, j;
  char intBuffer[32];
  char c;

  if(this->state != 0x00) return;

  this->state = 0x01;
  this->loader.loadRaw(&this->buffer);
  this->state = 0x02;

  // Parse header data.
  i = j = 0;
  width = -1, height = -1, fontSize = -1;
  while(true) {
    c = this->buffer[i++];
    if(c == '|') {
      intBuffer[j] = '\0';
      if(width == -1) {
        this->width = atoi(intBuffer);
        j = 0;
        continue;
      } else if(height == -1) {
        this->height = atoi(intBuffer);
        j = 0;
        continue;
      } else {
        fontSize = atoi(intBuffer);
        break;
      }
    }
    intBuffer[j++] = c;
  }

  this->state = 0x03;
  this->font.fontSize = fontSize;
  this->pixels = (struct Color*)(this->buffer + i);
  this->characterData = (truetypechar_t*)(
    (uint8_t*)this->pixels + (this->width * this->height * sizeof(struct Color))
  );
  this->state = 0x04;
}

TrueTypeAsset::~TrueTypeAsset() {

}