// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "UILabel.hpp"

using namespace Dawn;

UILabel::UILabel(UICanvas &canvas) : UIComponent(canvas) {

}

void UILabel::updatePositions() {
  UIComponent::updatePositions();
  this->updateMesh();
}

void UILabel::updateMesh() {
  if(!this->needsRebuffering) return;
  if(this->font == nullptr) return;

  this->font->buffer(
    this->text,
    this->fontSize,
    this->getWidth(),
    this->mesh,
    &this->measure
  );

  this->needsRebuffering = false;
}

std::string UILabel::getText() {
  return this->text;
}

void UILabel::setFont(Font *font) {
  this->font = font;
  this->needsRebuffering = true;
}

void UILabel::setText(std::string text) {
  this->text = text;
  this->needsRebuffering = true;
}

void UILabel::setFontSize(float_t fontSize) {
  this->fontSize = fontSize;
  this->needsRebuffering = true;
}

void UILabel::drawSelf(UIShader &shader, glm::mat4 selfTransform) {
  if(this->font == nullptr) return;

  this->updateMesh();
  shader.setUIColor(this->textColor);
  shader.setUIModel(selfTransform);
  shader.setUITexture(&this->font->getTexture());
  
  this->font->draw(this->mesh, this->startQuad, this->endQuad);
}