// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/_RenderManager.hpp"
#include "display/BackBufferRenderTarget.hpp"
#include "display/shader/SimpleTexturedShader.hpp"

namespace Dawn {
  class StandardRenderPipeline;

  class RenderManager : public IRenderManager {
    private:
      std::shared_ptr<StandardRenderPipeline> standardRenderPipeline;

    public:
      BackBufferRenderTarget backBuffer;
      std::shared_ptr<SimpleTexturedShader> simpleShader;
      std::shared_ptr<UIShader> uiShader;

      /**
       * Construct a new RenderManager for a game instance.
       */
      RenderManager(DawnGame &game);
      
      RenderTarget & getBackBuffer() override;
      RenderPipeline & getRenderPipeline() override;
      std::shared_ptr<Shader> getDefaultShader() override;
      std::shared_ptr<UIShader> getUIShader() override;
      void setRenderFlags(renderflag_t renderFlags) override;
      void init() override;
      void update() override;
      
      /**
       * Destroy a previously initialized RenderManager.
       */
      ~RenderManager();
  };
}