// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "DawnPokerGame.hpp"
#include "asset/assets/TextureAsset.hpp"
#include "asset/assets/TrueTypeAsset.hpp"
#include "visualnovel/ui/VisualNovelTextbox.hpp"

using namespace Dawn;

std::shared_ptr<TrueTypeAsset> assetFont;
std::shared_ptr<TextureAsset> assetTexture;

DawnGame::DawnGame(DawnHost &host) :
  host(host),
  renderManager(*this),
  inputManager(*this)
{
}

int32_t DawnGame::init() {
  this->assetManager.init();
  this->renderManager.init();

  this->scene = std::make_shared<Scene>(*this);

  auto cameraObject = this->scene->createSceneItem();
  auto camera = cameraObject->addComponent<Camera>();
  camera->transform.lookAt(glm::vec3(50, 50, 50), glm::vec3(0, 0, 0));
  
  auto canvas = UICanvas::createCanvas(this->scene);
  
  assetFont = this->assetManager.load<TrueTypeAsset>("truetype_ark");
  assetTexture = this->assetManager.load<TextureAsset>("texture_test");
  while(!assetFont->loaded || !assetTexture->loaded) {
    this->assetManager.update();
  }

  auto textbox = canvas->addElement<VisualNovelTextbox>();
  textbox->border.texture = assetTexture->texture.get();
  textbox->label.setFont(&assetFont->font);
  textbox->label.setFontSize(40);
  textbox->setTransform(
    UI_COMPONENT_ALIGN_STRETCH, UI_COMPONENT_ALIGN_END,
    glm::vec4(0, 250, 0, 0), 0.0f
  );

  // auto sprite = canvas->addElement<UISprite>();
  // sprite->setTransform(
  //   UI_COMPONENT_ALIGN_START,
  //   UI_COMPONENT_ALIGN_START,
  //   glm::vec4(0, 0, 200, 200),
  //   0
  // );
  // sprite->texture = &asset->font.getTexture();

  return DAWN_GAME_INIT_RESULT_SUCCESS;
}

int32_t DawnGame::update(float_t delta) {
  this->assetManager.update();
  this->inputManager.update();
  this->timeManager.update(delta);

  if(this->scene != nullptr) this->scene->update();
  
  this->renderManager.update();

  return DAWN_GAME_UPDATE_RESULT_SUCCESS;
}

DawnGame::~DawnGame() {
  
}